package gui.util.tablemodel;

import java.util.Vector;

import javax.swing.table.DefaultTableModel;

public class IdTableModel extends DefaultTableModel {

	private static final long serialVersionUID = 1L;

	private final int ID_COLUMN=0;
	
	public IdTableModel(String... columnNames) {
		addColumn("id");
		for (String columnName:columnNames)	addColumn(columnName);
	}
	
	
	synchronized public int getRowById(int id) {
		
		for (int i=0;i<getRowCount();i++) {
			if ((Integer)getValueAt(i, ID_COLUMN)==id) {
				return i;
			}
		}
		return -1;
	}

	@Override
	public void setColumnCount(int columnCount) {
		if (columnCount>0) super.setColumnCount(columnCount);
	}
	
	@Override
	public synchronized int getRowCount() {
		return super.getRowCount();
	}
	
	synchronized public void removeRowById(int id) {
		
		int row=getRowById(id);
		if (row>-1 && row<getRowCount() ) {
			removeRow(row);
		}
	}
	
	synchronized public void setData(int id,Object...data) {
		int row=getRowById(id);
		if (row>-1 && row<getRowCount() ) {
			for (int i=0;i<data.length;i++) {
				if (data[i]!=null) setValueAt(data[i], row, i+1);
			}
		}
	}
	
//	synchronized public void setDataAtColumn(int id,Object data,String col) {
//		int row=getRowById(id);
//		if (row>-1 && row<getRowCount() ) {
//			setValueAt(data, row, findColumn(col));
//		}
//	}
	
	synchronized public void addRow(int id,Object...data) {
		Vector<Object> v=new Vector<Object>();
		v.add(id);
		
		for (int i=0;i<data.length;i++)	v.add(data[i]);
		for (int i=data.length;i<getRowCount();i++) v.add(null);
		super.addRow(v);
	}
	
	synchronized public void removeAll() {
		setRowCount(0);
	}
	
	@Override
	public synchronized void removeRow(int row) {
		if (row<getRowCount())
		super.removeRow(row);
	}
	
	@Override
	public boolean isCellEditable(int row, int column) {
		return false;
	}
	
	@SuppressWarnings("unchecked")
	@Override
	public synchronized void setValueAt(Object aValue, int row, int column) {
		 
		 Vector<Object> rowVector = (Vector<Object> )dataVector.elementAt(row);
	     rowVector.setElementAt(aValue, column);
	     fireTableCellUpdated(row, column);
	}
	
	@SuppressWarnings("unchecked")
	@Override
	public synchronized Object getValueAt(int row, int column) {
		while (row>=getRowCount()) row--;
		if (row>-1) {
			 Vector<Object>  rowVector = (Vector<Object> )dataVector.elementAt(row);
		     return rowVector.elementAt(column);
		}else {
			return null;
		}
	}
	
	synchronized public int getId(int row) {
		if (row==-1) {
			return -1;
		} else {
		
			try  {
				return (Integer)getValueAt(row, ID_COLUMN);
				
			} catch (NumberFormatException nfe) {
				return -1;
			}
		}
	}
	
}
